import * as React from "react";
import { mount, ReactWrapper } from "enzyme";
import { matchers } from "jest-emotion";
expect.extend(matchers);

import SquareLoader from "../src/SquareLoader";
import { LoaderSizeProps } from "../src/interfaces";
import { sizeDefaults } from "../src/helpers";

describe("SquareLoader", () => {
  let loader: ReactWrapper;
  let props: LoaderSizeProps;
  let defaultSize: number = 50;
  let defaultColor: string = "#000000";

  it("should match snapshot", () => {
    loader = mount(<SquareLoader />);
    expect(loader).toMatchSnapshot();
  });

  it("should contain default props if no props are passed", () => {
    props = loader.props();
    expect(props).toEqual(sizeDefaults(defaultSize));
  });

  it("should contain styles created using default props", () => {
    expect(loader).toHaveStyleRule("height", `${defaultSize}px`);
    expect(loader).toHaveStyleRule("width", `${defaultSize}px`);
    expect(loader).toHaveStyleRule("background-color", defaultColor);
  });

  it("should render null if loading prop is set as false", () => {
    loader = mount(<SquareLoader loading={false} />);
    expect(loader.isEmptyRender()).toBe(true);
  });

  it("should render the correct color based on props", () => {
    let color: string = "#e2e2e2";
    loader = mount(<SquareLoader color={color} />);
    expect(loader).not.toHaveStyleRule("background-color", defaultColor);
    expect(loader).toHaveStyleRule("background-color", color);
  });

  it("should render the correct size based on props", () => {
    let size: number = 21;
    loader = mount(<SquareLoader size={size} />);

    expect(loader).not.toHaveStyleRule("height", `${defaultSize}px`);
    expect(loader).not.toHaveStyleRule("width", `${defaultSize}px`);

    expect(loader).toHaveStyleRule("height", `${size}px`);
    expect(loader).toHaveStyleRule("width", `${size}px`);
  });

  it("should render the css override based on props", () => {
    loader = mount(<SquareLoader css={"position: absolute; color: blue; display: block"} />);

    expect(loader).not.toHaveStyleRule("display", "inline-block");
    expect(loader).toHaveStyleRule("display", "block");
    expect(loader).toHaveStyleRule("position", "absolute");
    expect(loader).toHaveStyleRule("color", "blue");
  });
});
